#!/bin/bash

accumsize=16
declare -a  candidates=( 14  2  7  5 \
                         14  2  7  6 \
                         13  5 10 10 \
                          3  2  6  5 \
                          6  2  3  9 \
                          6  2 11  6 \
                          6  2  5  5 \
                          6  2  5  9 \
                          5  2  6  9 )

PRoptions="stdin -multithreaded -te 1 -tf 2 -tlmin 1KB"
#PRoptions="stdin -multithreaded -te 1 -tf 2 -tlmin 1KB -tlmaxonly"
PRcommand="./RNG_test.unchanged ${PRoptions}"
#PRcommand="./RNG_test.FPFless ${PRoptions}"



if [ $accumsize -lt 8 ]; then
	echo Aborted!  Specify size range 8-64.
	echo
	exit
fi

scoredir=test-s${accumsize}

if [ ! -d "${scoredir}" ]; then
	mkdir "${scoredir}"
fi



printf "\nBEGIN  ... at "
date +"%F %T"
printf "\nFrequency distribution reports of pairs and zero-runs for Xoroshiro$(($accumsize*2))($accumsize)++ random generator.\nWithout parity replacement bit.\n"
printf "PractRand v0.93 options:  ${PRoptions}\n\n"

top=${#candidates[*]}   # number of entries in array
enoughram=$((5 * 2**20))   # amount of spare ram required before running next test case
count=0
while [ $count -lt $top ]
do
	while true
	do
		sleep 2
		spareram=$(awk '/MemAvailable/{print $2}' /proc/meminfo)
		if [ $spareram -gt $enoughram ]; then  # Need more than 4 GB spare for next case
			break
		fi
	done

	a=${candidates[$count]}; count=$(($count + 1))
	b=${candidates[$count]}; count=$(($count + 1))
	c=${candidates[$count]}; count=$(($count + 1))
	d=${candidates[$count]}; count=$(($count + 1))

	printf "Test case Xoroshiro32++ [$a $b $c $d]\n"

# Compile candidate test case
	gcc -o "${scoredir}/testcase-dist-a${a}b${b}c${c}d${d}" "src/dist-s16.c" -O3 -DCONSTANT_A=${a} -DCONSTANT_B=${b} -DCONSTANT_C=${c} -DCONSTANT_D=${d}

	if [ $? -ne 0 ]; then
		echo Xoroshiro test case aborted!  Compile error.
		echo
		exit 1
	fi

# Run test cases, concurrently
	if [ $count -lt $top ]; then
		nice -11  "${scoredir}/testcase-dist-a${a}b${b}c${c}d${d}" &
	else
		nice -11  "${scoredir}/testcase-dist-a${a}b${b}c${c}d${d}"  # wait for completion before deleting the binaries
	fi
done

count=0
while [ $count -lt $top ]
do
	a=${candidates[$count]}; count=$(($count + 1))
	b=${candidates[$count]}; count=$(($count + 1))
	c=${candidates[$count]}; count=$(($count + 1))
	d=${candidates[$count]}; count=$(($count + 1))
	rm "${scoredir}/testcase-dist-a${a}b${b}c${c}d${d}"
done
